#!/bin/bash
# ex: set softtabstop=4

# GLOBAL STATIC Definitions

# The server whos archive this data will be migrated to
# MUST have medsrv ssh keys setup to $PEER  if not using localhost
PEER="localhost"

# The MySQL Dump program
MYSQLDUMP="$HOME/component/mysql/bin/mysqldump -t imagemedical Archive"
SQLDUMPDIR="$HOME/work/ArchiveMigration/sql"
if [ ! -e "$SQLDUMPDIR" ] ; then
   mkdir -p $SQLDUMPDIR
fi

# The RATE, given as a number representing the number of files
# to copy before sleeping, and SLEEPTIME is the number of seconds
# to sleep during a pause
RATE=600
SLEEPTIME=2

# This is the DB alias
DB="$HOME/component/mysql/bin/mysql -N imagemedical"

#  Archive base, assumes all archives are mounted under
#  $ARCHBASE
ARCHBASE="/home/medsrv/var/archive"

###########################################################
#  Functions
###########################################################

function logit()
{
  local str="$1"
  now=`date +"%b %d %Y %R:%S"`
  echo -e "$now $str"
} # end of function logit


function usage()
{
    echo -e  "\nUSAGE:"
    echo -e  "\n$0 -d DESTINATION -s SOURCE [-p PEER] [-h]"
    echo -e  "\t\nWHERE:\n"
    echo -e  "\t SOURCE is the full path to the mount point to be migrated,"
    echo -e  "\t\t i.e. /home/medsrv/var/archive/a2"
    echo -e  "\t DESTINATION is the full path of the destination mount point,"
    echo -e  "\t\t i.e. /home/medsrv/var/archive/archive3"
    echo -e  "\t PEER is the remote machine on which Destination resides,"
    echo -e  "\t\t if not set, PEER defaults to localhost.  Note ssh keys"
    echo -e  "\t\t must be set up as the medsrv user from the local machine"
    echo -e  "\t\t to the PEER machine if not using localhost"
    echo -e  "\n"
}


function updatePeerDB()
{
    local sqlfile="$1"
    cat $SQLDUMPDIR/$sqlfile
    if [ "$PEER" = "localhost" ] ; then
        rsync -va $SQLDUMPDIR/$sqlfile /home/medsrv/work/
        RES=$?
        if [ $RES -ne 0 ] ; then
           echo "updatePeerDB: rsync returned $RES exiting"
           return 123
        fi
        /home/medsrv/work/readsql.sh /home/medsrv/work/$sqlfile
        RES=$?
        if [ $RES -ne 0 ] ; then
           echo "updatePeerDB: ssh readsql returned $RES exiting"
           return 129
        fi
    else
        rsync -va -e ssh $SQLDUMPDIR/$sqlfile $PEER:/home/medsrv/work/
        RES=$?
        if [ $RES -ne 0 ] ; then
           echo "updatePeerDB: rsync returned $RES exiting"
           return 123
        fi
        ssh -n $PEER /home/medsrv/work/readsql.sh /home/medsrv/work/$sqlfile
        RES=$?
        if [ $RES -ne 0 ] ; then
           echo "updatePeerDB: ssh readsql returned $RES exiting"
           return 129
        fi
    fi
} # end updatePeerDB function


function migrateStudy()
{
    local LocalArchDir="$1"
    [ -z "$LocalArchDir" ] && logit "\tmigrateStudy: LocalArchDir missing!!" && return 1
    local study="$2"
    [ -z "$study" ] && logit "\tmigrateStudy: study missing!!" && return 2
    local rate=$3
    [ -z "$rate" ] && logit "\tmigrateStudy: rate missing using default of $RATE" 

    RemoteArchDir=`echo $LocalArchDir | sed -e "s!$MountPointSRC!$MountPointDST!"`
    ## TESTING ##
    ##echo "LocalArchDir  = $LocalArchDir"
    ##echo "RemoteArchDir = $RemoteArchDir"
    ##echo "study         = $study"
    ##echo "rate          = $rate"
    ## END OF TESTING ##
 
    if [ "$PEER" = "localhost" ] ; then
        #  We are moving from one local partition to another
        logit "\tMoving from one local partition to another"
        mkdir -p $ARCHBASE/$RemoteArchDir/$study
        RC=$?
        if [ $RC -ne 0 ] ; then
            logit "\tmkdir returned $RC ... failed to make directory"
            return 7
        fi
    else
        #  We are moving from one local partition to a remote partition
        logit "\tMoving from one local partition to a remote partition"
        ssh $PEER mkdir -p $ARCHBASE/$RemoteArchDir/$study/
        RC=$?
        if [ $RC -ne 0 ] ; then
            logit "\tmkdir returned $RC ... failed to make directory"
            return 7
        fi
    fi

    count=0
    for o in `ls $ARCHBASE/$LocalArchDir/$study`
    do
        mod=$((count % $rate))
        #echo "modulo: $mod"
        if [ $mod -eq 0 -a  $count -ne 0 ] ; then
           logit "\tSleeping $SLEEPTIME seconds..."
           sleep $SLEEPTIME
        fi
        logit "\tCopying: $o "
        logit "\t\tFROM: $ARCHBASE/$LocalArchDir/$study"
        logit "\t\tTO: $PEER:$ARCHBASE/$RemoteArchDir/$study"
        if [ "$PEER" = "localhost" ] ; then
            echo "rsync -va  $ARCHBASE/$LocalArchDir/$study/$o $ARCHBASE/$RemoteArchDir/$study/"
            rsync -va  $ARCHBASE/$LocalArchDir/$study/$o $ARCHBASE/$RemoteArchDir/$study/
            rc=$?
            if [ $rc -ne 0 ] ; then
               logit "\tcp returned $rc ... failed to sync $s / $o"
               return 8
            fi
            if [ -e $ARCHBASE/$LocalArchDir/$study/.info ] ; then
                echo "rsync -va $ARCHBASE/$LocalArchDir/$study/.info $ARCHBASE/$RemoteArchDir/$study/"
                rsync -va $ARCHBASE/$LocalArchDir/$study/.info $ARCHBASE/$RemoteArchDir/$study/
                rc=$?
                if [ $rc -ne 0 ] ; then
                   logit "\tcp returned $rc ... failed to sync $study / .info"
                   return 9
                fi
            fi
        else
            echo "rsync -va -e ssh $ARCHBASE/$LocalArchDir/$study/$o $PEER:$ARCHBASE/$RemoteArchDir/$study/"
            rsync -va -e ssh $ARCHBASE/$LocalArchDir/$study/$o $PEER:$ARCHBASE/$RemoteArchDir/$study/
            rc=$?
            if [ $rc -ne 0 ] ; then
               logit "\tcp returned $rc ... failed to sync $s / $o"
               return 8
            fi
            if [ -e $ARCHBASE/$LocalArchDir/$study/.info ] ; then
                echo "rsync -va -e ssh  $ARCHBASE/$LocalArchDir/$study/.info $PEER:$ARCHBASE/$RemoteArchDir/$study/"
                rsync -va -e ssh  $ARCHBASE/$LocalArchDir/$study/.info $PEER:$ARCHBASE/$RemoteArchDir/$study/
                rc=$?
                if [ $rc -ne 0 ] ; then
                   logit "\tcp returned $rc ... failed to sync $study / .info"
                   return 9
                fi
            fi
        fi
        count=$((count+1))
    done 
    

    $MYSQLDUMP -w "STYIUID='${study}'" > $SQLDUMPDIR/$study.sql
    rc=$?
    if [ $rc -ne 0 ] ; then
       logit "\t MySQL Dump returned $rc ... $study "
       return 11
    fi
    
    if [ "$PEER" != "localhost" ] ; then
        cp  $SQLDUMPDIR/$study.sql $SQLDUMPDIR/$study.sql.ORIG
        cat $SQLDUMPDIR/$study.sql.ORIG | sed -e "s!$LocalArchDir!$RemoteArchDir!" > $SQLDUMPDIR/$study.sql
    fi
    
    updatePeerDB ${study}.sql
    rc=$?
    if [ $rc -ne 0 ] ; then
       logit "\t updatePeerDB returned $rc ... $study "
       return 12
    fi

    return 0
} # end migratestudy function



###########################################################
#    MAIN 
###########################################################


###########################################################
#  Parse Command Line Arguments
###########################################################
while getopts ":p:hd:s:" optionName
do
   case $optionName in 
   p)  PEER=$OPTARG 
          ;; # Process PEER Option
   d)  DESTINATION=$OPTARG 
          ;; # Process Destination Directory
   s)  SOURCE=$OPTARG 
          ;; # Process Source Directory
   h)  usage 
          exit 0 
          ;; # Print Help and exit
   *)  echo -e "\nERROR: Unknown option [ $OPTARG ]" 
          usage 
          exit 1 
          ;;
   esac
done

# This is the mount point, i.e. a2
MountPointSRC=`basename $SOURCE`
MountPointDST=`basename $DESTINATION`

# This is the full path minus the mount point, i.e. /home/medsrv/var/archive
MountPathSRC=`dirname $SOURCE`
MountPathDST=`dirname $DESTINATION`

# File listing successfully moved studies
GOODFILE="$MountPointSRC.moved"

# File listing successfully moved studies
BADFILE="$MountPointSRC.failed"

logit "\tSource:        $SOURCE"
logit "\tDestination:   $DESTINATION"
logit "\tMountPointSRC: $MountPointSRC"
logit "\tMountPointDST: $MountPointDST"
logit "\tMountPathSRC:  $MountPathSRC"
logit "\tMountPathDST:  $MountPathDST"
STUDYFILE=${MountPointSRC}.stys
logit "\tSTUDYFILE: $STUDYFILE"

if [ ! -e "$STUDYFILE" ] ; then
  logit "\tFinding Studies on $SOURCE"
  echo "select ARCHDIR, STYIUID from Archive where ARCHDIR like '${MountPointSRC}%' " | $DB > ${MountPointSRC}.tmp
  while read archdir styiuid
  do
      echo "/home/medsrv/var/archive/${archdir}/${styiuid}" >> ${MountPointSRC}.stys
  done < ${MountPointSRC}.tmp
else
  logit "\t$STUDYFILE already exists, skipping the find..."
fi

#  Loop through study list and migrate them
#for s in `cat $STUDYFILE | head -n 10 `
for s in `cat $STUDYFILE `
do
    BNAME=`echo $s | sed -e "s!$MountPathSRC/!!"`
    ARCHDIR=`dirname $BNAME`
    STY=`basename $BNAME`
    logit "\n-------------------------------------------------"
    logit "\tStudy:   $STY"
    logit "\tBNAME:   $BNAME"
    logit "\tARCHDIR: $ARCHDIR"

    # Migrate the study
    migrateStudy $ARCHDIR $STY $RATE 
    RES=$?
    if [ $RES -eq 0 ] ; then
        logit "\t$STY Study Migration successful"
        if [ -e ${SQLDUMPDIR}/${STY}.sql ] ; then
          echo "delete from Archive where STYIUID='${STY}'" | $DB
        fi
        echo "$STY" >>$GOODFILE
    else
        logit "\t$STY Study Migration NOT successful. Return Code: $RES"
        echo "$STY" >>$BADFILE
    fi
done

logit "\n$0 Finished. \nSee $GOODFILE for list of successfully moved studies"
logit "See $BADFILE for list of studies which were not moved"
