#!/bin/sh
#
# Description:
#	Custom Script for Multi Mobil only (will not work on other systems) to compute total time a technicain takes for each study.
#
# Usage:
#	get_techtime.sh <STYIUID> <MODALITY TYPE>
#
# Output:
#	database feilds:
#		Study Date, Technician's Name (USER1), Referring Physician, Study Description 
#	calculated data:
#		Time in minutes it takes Technican to perfrom procedure based on DICOM Content time and date.		
#
# Author: Bryan K. DeLaney 1/24/2014
#

. $HOME/etc/setenv.sh
. $TOOLS_HOME/functions.shi


study_STYIUID="$1"
study_MODALITY="$2"

study_directory="$(getStudyDirectory_ $study_STYIUID)"
if [ "$?" -ne 0 ]; then
	exit;
fi

# Loop through all modality files in the study directroy that start with Modality Type (i.e. US.1.2.3.4.5) 
# Use dcmdump and grep for content time and date. 
# DICOM Standard allows mulitple formats, use sed to normalize data 

for file in $( ls $study_directory/$study_MODALITY* )
do
  ~/component/dcmtk/bin/dcmdump $file > $HOME/tmp/output.txt 2> $HOME/tmp/get_techtime.error
  
  if [ "$?" -eq 0 ]; then 
  	content_DATE=$(cat $HOME/tmp/output.txt | grep "ContentDate" | awk '{print $3}' | sed -e 's/\[//' | sed -e 's/\]//' | sed -e 's/\://g'| awk -F '.' '{print $1}')
  	content_TIME=$(cat $HOME/tmp/output.txt | grep "ContentTime" | awk '{print $3}' | sed -e 's/\[//' | sed -e 's/\]//' | sed -e 's/\://g'| awk -F '.' '{print $1}')
  	time_HH=${content_TIME:0:2}
  	time_MM=${content_TIME:2:2}
  	time_SS=${content_TIME:4:2}
  	date_CCYY=${content_DATE:0:4}
  	date_MM=${content_DATE:4:2}
  	date_DD=${content_DATE:6:2}

  	echo $(date +%s -d "$date_MM/$date_DD/$date_CCYY $time_HH:$time_MM:$time_SS") >> $HOME/tmp/$study_STYIUID.get_techtime.tmp
  else
	echo "Can't dump $file in Study $study_STYIUID" >> $HOME/tmp/get_techtime.error
  fi
done

start_seconds=$(cat $HOME/tmp/$study_STYIUID.get_techtime.tmp | sort | head -1)
end_seconds=$(cat $HOME/tmp/$study_STYIUID.get_techtime.tmp | sort | tail -1)
total_seconds=$(expr $end_seconds - $start_seconds)

# Output is tab delimited use awk to change to "|" delmited. REFPHNAME fields contains dicom ^ delimiter use sed to strip out 

database_info=$(echo "SELECT STYDATE,USER1,REFPHNAME,STYDESCRIPTION FROM Dcstudy WHERE STYIUID='$study_STYIUID';" | ~/component/mysql/bin/mysql imagemedical -N | awk -F '\t' '{print $1"|"$2"|"$3"|"$4"|"}'|sed -e 's/\^/, /'|sed -e 's/\^\^\^/ /')

echo -n $database_info
echo "scale=1;$total_seconds / 60" | bc

rm $HOME/tmp/$study_STYIUID.get_techtime.tmp
rm $HOME/tmp/output.txt

if [ ! -s $HOME/tmp/get_techtime.error ]; then

	rm $HOME/tmp/get_techtime.error
fi
