#!/bin/bash
#######################################################################################
#  JobController.sh - simple job controller script.  Typically used to control the
#                     operation of a long running script for several days.  Ensuring
#                     the job is running during 'off hours' and is suspended during
#                     'on hours'.
#  Author           -  Tony Bowles
#  Date             - Jan 8, 2009
#  Usage            - ./JobController.sh
#  Note             - Change the JobToControl variable to point to the script that
#                     you wish for this controller to control
#######################################################################################

JobToControl="./validateStudies.sh -s eRAD_DR > validateOutputFile"
#JobToControl="./test.sh"

longSleeptime=600
quickSleeptime=300

BusinessStartTime="08"
BusinessEndTime="21"
CurrentHour=`date +%H`

# Loop for controlling the child job
cntrlpid="xxxx"

while [ 1 ]
do
   echo "Main Loop: Sleeping $quickSleeptime Seconds..."
   sleep $quickSleeptime

   ThePid=`ps aux | grep $JobToControl | grep -v grep | awk '{ print $2 }'`
   CurrentHour=`date +%H`

#   # For testing only...
#   minute=`date +%M`
#   echo "Minute = $minute"
#   rc=`expr $minute % 2 `
#   if [ $rc -eq 0 ] ; then

   if [ "$CurrentHour" -lt "$BusinessStartTime" -o "$CurrentHour" -gt "$BusinessEndTime" ] ; then # its ok to run
      # It is RUNTIME, the job should be running
      if [ "$ThePid" != "" ] ; then # the job is already running
         echo "RUNTIME: the job, $JobToControl, is already active with PID = $ThePid"
         taskStatus=`ps aux | grep $JobToControl | grep -v grep | awk '{ print $8 }'`
         echo "RUNTIME: taskStatus = $taskStatus"
         if [ "$taskStatus" = "T+"  ] ; then # the job is running but suspended, start it
            echo "RUNTIME: the job, $ThePid, is active, but suspended. Sending SIGSTART." 
            kill -s SIGCONT $ThePid
         else
	    echo "RUNTIME: the job, $ThePid, is active and running.  Nothing to do"
         fi
         continue
      else
         echo "RUNTIME: The job is not yet running starting it"
         $JobToControl & cntrlpid=$!
      fi
   else
      taskStatus=`ps aux | grep $JobToControl | grep -v grep | awk '{ print $8 }'`
      echo "Nonruntime: taskStatus = $taskStatus"
      if [ "$ThePid" = "$cntrlpid"  ] ; then # the job is already running
         # Send $ThePid the control Z signal
         echo "Nonruntime: The job is active, is it running or suspended?"
         if [ "$taskStatus" != "T+" ] ; then
             echo "Nonruntime: The job is running. We need to suspend it. Sending SIGSTOP."
             kill -s SIGSTOP $ThePid
         else
             echo "Nonruntime: The job is already suspended. Nothing to do"
         fi
      else
         # During business (nonruntime) hours and the job is not running.  good.
         echo "Nonruntime: The job is not active at all (running or suspended)."
      fi 
      # It is Nonruntime, don't run, just sleep
      echo -e "Nonruntime: The Current Hour is $CurrentHour. Sleeping $longSleeptime seconds..."
      sleep $longSleeptime
   fi
done


