#/bin/sh
#
# Look at each object in a Study and modify the specified DICOM tag if appropriate
#
# Usage: ModifyDicomTag.sh <STYIUID> <DICOMTAG> <CURRENTVALUE> <NEWVALUE>
# 
# 
# <STYIUID> is the Study Instance UID
# <DICOMTAG> is a non Sequence DICOM tag
# <CURRENTVALUE> is the current content of the tag
# <NEWVALUE> is the value that will be written into the tag if <CURRENTVALUE> matches current content of tag
# Example: ./ModifyDicomTag.sh 1.2.124.113532.25.47565.9271.20120803.121036.20426753  "(0008,0064)" "WSD" "SI"
# Output logged to /home/medsrv/tmp/taglog_<DATE> (i.e. taglog_2012-08-13
# Author: Bryan K. DeLaney 8/12/2012 Version 1.0

if [ "$#" -ne 4 ]
then
	echo "Look at each object in a Study and modify the specified DICOM tag if appropriate"
	echo ""
	echo "Usage: ModifyDicomTag.sh <STYIUID> <DICOMTAG> <CURRENTVALUE> <NEWVALUE>"	
	echo ""
	echo "Example: ./ModifyDicomTag.sh 1.2.124.113532.25.47565.9271.20120803.121036.20426753  \"(0008,0064)\" \"WSD\" \"SI\""
	echo ""
        echo "<STYIUID> is the Study Instance UID"
	echo "<DICOMTAG> is a non Sequence DICOM tag>"
	echo "<CURRENTVALUE> is the current content of the tag"
	echo "<NEWVALUE> is the value that will be written into the tag if <CURRENTVALUE> matches current content of tag"
	echo "Output logged to /home/medsrv/tmp/taglog_<DATE> (i.e. taglog_2012-08-13)"
else
	LOGDATE=`date +%F`
	LOGFILE="/home/medsrv/tmp/taglog_$LOGDATE"
	STUDY=$1
	TAG=$2
	OLDTAGVALUE=$3
	NEWTAGVALUE=$4
	STYDIR=`/home/medsrv/component/repositoryhandler/scripts/locateStudy.sh -d $STUDY`
	OBJECTCOUNT=`ls -1 $STYDIR/* | wc -l`


	if [ "$OBJECTCOUNT" -ge  1 ]
	then
		if [ -f /home/medsrv/tmp/fix_tag.lst ]
		then	
			rm /home/medsrv/tmp/fix_tag.lst
		fi

   		ls -1 $STYDIR/* >> /home/medsrv/tmp/fix_tag.lst

		echo "Evaluating $STUDY" >> $LOGFILE

		for LONGFILENAME in $(cat /home/medsrv/tmp/fix_tag.lst)
		do
			VERIFYTAG=`/home/medsrv/component/dcmtk/bin/dcmdump $LONGFILENAME | grep "$TAG" | awk '{print $3}' | sed 's/\[//g' | sed 's/\]//g'`
			if [ "$VERIFYTAG" = "$OLDTAGVALUE" ]
			then
				OBJECTFILE=`basename $LONGFILENAME`
				~/component/dcviewer/dcmod -t "$TAG" -v "$NEWTAGVALUE" $STYDIR/$OBJECTFILE
				~/component/utils/bin/storeDICOMFile -s $STUDY -f $OBJECTFILE -r
				echo "$STUDY $OBJECTFILE tag $TAG was $OLDTAGVALUE now set to $NEWTAGVALUE" >> $LOGFILE
			fi
		done

                if [ -f /home/medsrv/tmp/fix_tag.lst ]
                then
                        rm /home/medsrv/tmp/fix_tag.lst
                fi
	fi
fi
