#!/bin/bash
######################################################################################################################
# Given the full path to a study, (not in a repository) check if that study exists in a repository and delete it
######################################################################################################################

# Source the environment setup
. $HOME/etc/setenv.sh

sourcedir=$1
skipcmp=$2

function removeArchiveTableEntry() {
    local styiuid=$1
    echo "delete from Archive where STYIUID='${styiuid}'" | $HOME/component/mysql/bin/mysql imagemedical
    [ $? -ne 0 ] && return 1 || return 0
} # end function backupArchiveTableEntry


if [ -z "$sourcedir" ] ; then 
    echo -e "\nERROR: Missing full path to study\n"
    echo -e "\n\tUsage: $0 fullpathtoarchivedirectory\n\n"
    exit 1
elif [ ! -d "$1" ] ; then
    echo -e "\nERROR: '$1' does not exist\n"
    echo -e "\n\tUsage: $0 fullpathtoarchivedirectory\n\n"
    exit 2
fi

styiuid=$(basename $sourcedir)
repodir=$($REPOSITORY_HANDLER_HOME/scripts/locateStudy.sh -d $styiuid)

if [ "${sourcedir}" == "${repodir}" ]; then
    echo -e "\nERROR: This script won't work on repository directories\n"
    exit 1
fi

echo -e "\nCurrent Study: $styiuid"

if [ -z "$repodir" ] ; then
   echo -e "\tError: $styiuid has no repository directory. Exiting.\n"
   exit 1
fi

sourcecount=$(ls $sourcedir  | wc -l)
repocount=$(ls $repodir | wc -l)

if [ "${sourcecount}" != "${repocount}" ]; then
    echo -e "\tThere are a different number of objects in $sourcedir"
    echo -e "\tthan there are in $repodir"

    if [ "${sourcecount}" -gt "${repocount}" ]; then
        echo "Error: There are more objects in $sourcedir than in $repodir, exiting"
        exit 1
    else
        echo "There are less objects in $sourcedir than in $repodir.. strange"
    fi
fi

# Check each Object in the Study
unset o
for o in $(ls $sourcedir); do
    echo -e "\tChecking object: $o"
    repoobj="${repodir}/${o}"
    sourceobj="${sourcedir}/${o}"

    # Begin tests here.  If a test fails exit with error

    # Does source object exist in repo?
    if [ ! -e "$repoobj" ] ; then
	echo -e "\t\tERROR: $sourceobj does not exist in $repodir"
	exit 1
    fi

    # Is source object the same as repo object?
    cmp $sourceobj $repoobj
    if [ $? -ne 0 ]; then
        echo "Error: source and repository objects are not the same"
        if [ "$skipcmp" == "skipcmp" ]; then
            echo "$0 called with 'skipcmp' argument, continuing anyway"
        else
            exit 1
        fi
    fi
done

# If we got here all tests were successful
echo -e "\tAll objects checked ok for $styiuid, removing from archive.\n"
removeArchiveTableEntry $styiuid
[ $? -ne 0 ] && { echo "Error removing archive table entry"; exit 1; }

rm -rf $sourcedir
[ $? -ne 0 ] && { echo "Error removing $sourcedir"; exit 1; } || exit 0


