#!/bin/bash

LISTFILE="realstudies.list"

myPWD=`pwd`

. $HOME/etc/setenv.sh 

# Find the studies with objects if LISTFILE doesn't already exist...
if [ ! -e $myPWD/$LISTFILE ] ; then
   echo "$myPWD/$LISTFILE doesn't exist finding studies, this can take a LONG time"
   find $HOME/data/dicom.hashed -type f -name "[A-Z][A-Z]*" > $myPWD/$LISTFILE.tmp
   cat $myPWD/$LISTFILE.tmp | grep -v PbR > $myPWD/$LISTFILE
else
   echo "$myPWD/$LISTFILE does exist"
fi

# Now that we have the findings list, we need to extract the STYIUD
for l in `cat $myPWD/$LISTFILE `  
do
    if [ "$l" = "" ] ; then
       continue
    fi

    # l look like /path/to/dir/study/objectfile
    echo "l = $l"    
    # dname will be /path/to/dir/study
    dname=`dirname $l`
    if [ "$dname" = "" ] ; then 
       continue
    fi

    # o will be objectfile
    o=`basename $l`
    if [ "$o" = "" ] ; then 
       continue
    fi

    # s will be study
    s=`basename $dname`
    if [ "$s" = "" ] ; then 
       continue
    fi

    # device will look like OwnerDevice="child"
    device=`$HOME/component/tools/printStudyInfo.sh -s $s -S | grep OwnerDevice`

    # hub will look like child
    hub=`echo $device | awk -F\" '{ print $2 }'`

    if [ "$hub" != "" -a "$s" != "" ] ; then
       # Get the archive directory from the hub server
       archloc=`ssh -n $hub /home/image/tools/q_archiveloc.sh $s`
       if [ "$archloc" = "/home/medsrv/var/archive/" ] ; then
          echo "ERROR: $s not archived on $hub!!"
          echo "$s $hub" >> NotOnArchive.stys
          continue
       fi

       # find this studies dicom directory on this server
       studyDir=`$HOME/component/repositoryhandler/scripts/locateStudy.sh -d $s`
       if [ $? -ne 0 ] ; then
          echo "Error in $HOME/component/repositoryhandler/scripts/locateStudy.sh -d $s call.  Continuing"
          continue
       fi

       # get the list of objects present on this server
       thisStudysObjects=`ls $studyDir`
       if [ "x$thisStudysObjects" = "x" ] ; then
          echo "Error: thisStudysObjects is blank.  This study, $s , has no objects on this machine"
          continue
       fi

       # determine if all objects are on the archive
       allObjsArchived="TRUE"
       for obj in `echo $thisStudysObjects`
       do
            if [ "x$obj" = "x" ] ; then
               echo "obj is blank, continuing..." 
               continue
            fi
            
            echo "ssh -n $hub ls $archloc/$obj "
            ssh -n $hub ls $archloc/$obj 2>>err >>ObjectCleanup.archivelist
            if [ $? -ne 0 ] ; then
                echo "Object, $obj , for study, $s ,  NOT on $hub archive.  Error!"
                allObjsArchived="FALSE"
                echo "rsync -va -e ssh $studyDir/$obj $hub:$archloc"
            else
                echo "Object, $obj , for study, $s ,  IS ON $hub archive, removing it."
                echo "$HOME/component/utils/bin/refcount -r database -i $s/$obj -t OBJECTFILE"
                $HOME/component/utils/bin/refcount -r database -i $s/$obj -t OBJECTFILE
                echo "refcount returned $?"
            fi
       done

       # if all objects present on the server are on the hubs archive, remove the study
       if [ "$allObjsArchived" = "TRUE" ] ; then 
            echo -e "All objects for $s present on the WL server on present in thier hubs archive.  Removing study"
            # Ok, pull the trigger
            $HOME/component/cases/rmstudy.sh $s
            /home/image/tools/checkrefs.sh -s $s -R -S -c
       else
            echo "Not all objects for $s have been archived to $hub , we can not remove this study"
            continue
       fi

    else 
       echo "Either hub is blank, hub: $hub or s is blank, s: $s"

    fi

done  

#rm -f $myPWD/$LISTFILE

