#! /usr/bin/python
# ex: set expandtab shiftwidth=4 softtabstop=4:

import sys
import os
import shutil
import re
import time
import string
import urllib

curl = '/usr/bin/curl'
curl_success = '#RESULT: OK\n'
inprocess = '-20'
completed = '-5'
unviewed = '0'
viewed   = '20'
debug = 'TRUE'
hl7pidfile= '/home/medsrv/var/run/hl7.pid'

# Check if HL7 is running
# Exit if it isn't

if not os.path.exists(hl7pidfile):
    print "HL7 is not running, exiting.."
    sys.exit(0)







# Notes -
# styiuid : qe_SYUI
# timestamp format: (tm_year,tm_mon,tm_day,tm_hour,tm_min,tm_sec,tm_wday,tm_yday,tm_isdst)
# End notes

qe_to_db = {'qe_PAID':'PID', 
            'qe_PANM':'PNAME',
            'qe_PDOB':'PBDATE',
            'qe_PSEX':'PSEX',
            'qe_ACCN':'ACCNO',
            'qe_PHRF':'REFPHNAME',
            'qe_MODA':'MODALITY',
            'qe_INSN':'INSTNAME',
            'qe_DSCR':'STYDESCR',
            'qe_UDF1':'USER1'}



#teststudy = '1.2.840.113619.2.55.3.313272698.966.1193116695.28'
queryunviewed = '"select STYIUID from Dcstudy where MAINST = \'%s\';"' % unviewed
#querytest = '"select STYIUID from Dcstudy where STYIUID = \'%s\';"' % teststudy


def getSecondsSinceEpoch():
   #datecommand = 'date +%s'
   #res = os.popen(datecommand).readline()[:-1]
   #return res
   res = str(int(time.time()))
   return res
# End of getSecondsSinceEpoch



def makeAccession(prefix,suffix):
    seed = getSecondsSinceEpoch()
    myaccno = prefix + seed + suffix
    return myaccno
# End of makeAccession



def querydb(sql_cmd, mysql_bin="/home/medsrv/component/mysql/bin/mysql", mysql_db="imagemedical"):

    """ Queries mysql database. The sql_cmd could be built roughly like this:
        sql_cmd = '"select ID, HOST, AE, TYPE, HOSTNAME, PORT, APPLICID, FACILITYID,\
                    RECVTO, SENDTO, PROTOCOL  from Target where type = \'%s\';"' % devicetype
    """
   
    cmd = "/bin/echo %s | %s -N %s" % (sql_cmd, mysql_bin, mysql_db)
    lQueryrows = os.popen(cmd).readlines()

    if lQueryrows: # Don't bother stripping linefeeds if the result set is empty
        lQueryrows = stripLinefeeds(lQueryrows)

    return lQueryrows 
# End of querydb()



def initStudyInfo():
    studyinfo = {'qe_PAID':'', 
                 'qe_PANM':'',
                 'qe_PDOB':'',
                 'qe_PSEX':'',
                 'cControl':'new',
                 'qe_ACCN':'',
                 'qe_PHRF':'',
                 'qe_MODA':'',
                 'qe_UDF1':'',
                 'qe_INSN':'',
                 'qe_DSCR':''}
    return studyinfo
# End of initStudyInfo()



def queryStudyInfo(styiuid):
    """ Given a styiuid, query Dcstudy for the values for keys listed in the dictionary 'qe_to_db'
        Returns a dictionary studyinfo with qe_* keys for use in jsp calls

        ** Note - If the database returns multiple results, only the first item is used
    """

    studyinfo = initStudyInfo()
    for k, v in qe_to_db.iteritems():
        sql_cmd = '"select %s from Dcstudy where styiuid = \'%s\';"' % (v, styiuid)
        # Should probably rewrite this to only query the db once 
        studyinfo[k] = querydb(sql_cmd)[0]

    if debug == 'TRUE':
        print "Debug: in queryStudyInfo()"
        for k, v in studyinfo.iteritems():
            print "%s : %s" % (k, v)
    return studyinfo

# End of queryStudyInfo



def stripLinefeeds(lines):
    rv = []
    for line in lines:
        if line[-1] == '\n':
            line = line[:-1]
        rv.append(line)
    return rv
# stripLinefeeds()



def createOrder(studyinfo, orderaccession):

    # call doModify later on the created order to set the accession number to what it is supposed to be

    curlargs = "https://127.0.0.1/iface/importOrder.jsp?cControl=%s\&qe_ACCN=%s\&qe_PANM=%s\&qe_PDOB=%s\&qe_PSEX=%s\&qe_PAID=%s\&qe_PHRF=%s\&qe_MODA=%s\&qe_INSN=%s\&qe_DSCR=%s\&qe_UDF1=%s" % (studyinfo['cControl'], orderaccession, studyinfo['qe_PANM'], studyinfo['qe_PDOB'], studyinfo['qe_PSEX'], studyinfo['qe_PAID'], studyinfo['qe_PHRF'], studyinfo['qe_MODA'], studyinfo['qe_INSN'], studyinfo['qe_DSCR'], studyinfo['qe_INSN'])

    cmd = "%s %s" % (curl, curlargs)
    print "%s" % cmd

    r = os.popen(cmd).readlines()
    print "%s" % r

    if curl_success in r:
        res = 'TRUE'
    else:
        res = 'FALSE'

    return res
# End of createOrder()


def doStatusModify(styiuid, newstatus):
    curlargs = "https://127.0.0.1/iface/editStudy.jsp?cStudy=%s\&cMode=Soft\&qe_STAT=%s 2>/dev/null" \
        % (styiuid, newstatus)
    print curlargs
    cmd = "%s %s" % (curl, curlargs)
    r = os.popen(cmd).readlines()
    print "%s" % r
    if curl_success in r:
        res = 'TRUE'
    else:
        res = 'FALSE'

    return res
# doStatusModify


def doModify(styiuid, qeflags):

    curlargs = "https://127.0.0.1/iface/editStudy.jsp?cStudy=%s\&cMode=Soft\&%s 2>/dev/null" \
        % (styiuid, qeflags)
    cmd = "%s %s" % (curl, curlargs)
    r = os.popen(cmd).readlines()
    print "%s" % r
    if curl_success in r:
        res = 'TRUE'
    else:
        res = 'FALSE'

    return res
# doModify

def curlIfy(dStudyinfo):

    """ Given a dictionary of study information dStudyinfo, replace spaces with '+' and
        escape '(' , '&', and ')' so that bash won't interpret them when we call curl

        There may be a better way to do this using string.translate
    """

    trch = (' ', '&', '(', ')', '\'', '[', ']', ) # Characters to escape or translate
    escapech = "\\"

    for k, v in dStudyinfo.iteritems(): # for each study info item
        for ch in trch: # loop through  of the list of characters we want to translate
            # If ch is a space, replace with a +
            # else escape with \
            if ch == ' ':
                newch = '+'
            else:
                newch = "%s%s" % (escapech, ch)
            v = v.replace(ch, newch)
            dStudyinfo[k] = v

    if debug == 'TRUE':
        print "Debug: in curlIfy()"
        for k, v in dStudyinfo.iteritems():
            print "%s : %s" % (k, v)

    return dStudyinfo

# End of curlIfy()

 
# Here we decide what studies to operate on
todo = querydb(queryunviewed)
#todo = querydb(querytest)

counter = 0
maxcounter = 9
if todo:
    print
    print "%s studies on worklist that match status %s" % (len(todo), unviewed)
    for s in todo:
        # Keep accession number suffix (counter) single digit
        if counter > maxcounter:
            counter = 0
        counter += 1
        print "Querying info for study %s at %s" % (s, time.asctime())
        sinfo = queryStudyInfo(s)
        # the following was for testing accno generation
        #myaccno = makeAccession('EP',str(counter))
        #print myaccno
        #sys.exit(0)
        if not sinfo['qe_ACCN']: # If the accession number is blank
            print "%s does not have an accession number, creating one" % s
            myaccession = makeAccession('EP',str(counter))
            sinfo['qe_ACCN'] = myaccession

        if sinfo['qe_PSEX'].upper() != 'M' and sinfo['qe_PSEX'].upper() != 'F' : # Patient sex is a required field
            print "%s was sent with no value in patient sex, or not M or F, setting to 'U'" % s
            sinfo['qe_PSEX'] = 'U'
    
        curlifiedstudyinfo = curlIfy(sinfo)

        print "Creating order for %s" % s
        orderaccession = makeAccession('OT',str(counter))
        res = createOrder(curlifiedstudyinfo, orderaccession)
        if res == 'FALSE':
            print "Error creating order for %s" % (s)
            print "Skipping "
            #print "Exiting with status 1"
            #sys.exit(1)
            continue
        else:
            print "Order created successfully for %s with accno %s" % (s, orderaccession)

        # Find the styiuid for the order we just created

        querycreatedorder = '"select STYIUID from Dcstudy where ACCNO = \'%s\';"' % orderaccession
        orderstyiuid = querydb(querycreatedorder)[0]
        print "styiuid for order %s is %s" % (orderaccession, orderstyiuid)

        # Modify the newly created order to have the correct accession number
        print "Setting ACCNO to %s on order %s" % (sinfo['qe_ACCN'], orderstyiuid)
        qeflags = "qe_ACCN=%s" % sinfo['qe_ACCN']
        res = doModify(orderstyiuid, qeflags)
        if res == 'FALSE':
            print "Error setting ACCNO to %s on order %s" % (sinfo['qe_ACCN'], orderstyiuid)
            print "Exiting with status 1"
            sys.exit(1)
        else:
            print "Successfully set ACCNO to %s on order %s" % (sinfo['qe_ACCN'], orderstyiuid)


        # Set the study to "viewed" so that we don't process it twice
        print "Setting %s to 'viewed'" % s
        res = doStatusModify(s, viewed)
        if res == 'FALSE':
            print "Error setting %s to viewed" % (s)
            #print "Exiting with status 1"
            print "Skipping"
            #sys.exit(1)
            continue
        else:
            print "Successfully set %s to 'viewed'" % (s)
        print

    # Call autocorrection (outside of the for loop)
    print "Starting autocorrection"
    autocorrectcmd = '/home/medsrv/component/taskd/runjavacmd -c "0 imagemed.dcprocessing.AutoCorrection" -p 5555'
    os.popen(autocorrectcmd)

else:
    if debug == 'TRUE':
        print "%s : No studies on the worklist match status %s" % (time.asctime(), unviewed)



   
    
