#!/bin/bash
#
#  Name:    forwardMissingObjstoInsite.sh
#  Author:  Tony Bowles
#  Date:    5/13/08 (Last modified)
#  Input:   $1 - Study List file, file containing STYIUIDs one per line of studies to check/forward
#  Description: Checks each object in data/dicom/STYIUID to see if it exists on insite, if not
#               it forwards that object and then rechecks.  Creates two output files, one to store
#               a list of studies successfully sent to insite (or already there) and one to store
#               a list of studies which still have objects missing from insite.
#

. $HOME/etc/setenv.sh
. $HOME/var/dicom/pb-scp.cfg

#  Output files
# Successful send list (stys)
SUCCESSFULSENDFILE="InsiteSentSuccesfully.stys"

# Unsuccessful send list (stys)
STILLMISSINGFILE="InsiteStillMissing.stys"

# You must verify these setting match the device
# information for the Insite storage device on this
# server before beginning.
InsiteAE="indexStore-P0CS"
InsiteIP="10.1.250.160"
InsitePort="10104"
OurAE="ERAD_PACS"

CheckInsiteScript="$HOME/component/checkintegrity/forwarded2Insite.sh"

[ -z "$1" ] && echo -e "\n\n\tMissing Argument: Study List\n\n" && exit 1

for s in `cat $1`
do
   ALLOBJECTSFWD="TRUE"
   for o in `ls $HOME/data/dicom/$s`
   do
     isForwarded=`$CheckInsiteScript $s $o | awk '{ print $1 }'`
     PBR=`ls ~/data/dicom/$s/ | grep PbR`
     echo -e "\nisForwarded = $isForwarded"

     if [ "$isForwarded" = "FAILED" ] ; then
        echo "We need to forward $o"
        echo "$HOME/component/dicom/bin/storescu -R -aet $OurAE -aec $InsiteAE -pbr $HOME/data/dicom/$s/$PBR $InsiteIP $InsitePort  $HOME/data/dicom/$s/$o"
        $HOME/component/dicom/bin/storescu -R -aet $OurAE -aec $InsiteAE -pbr $HOME/data/dicom/$s/$PBR $InsiteIP $InsitePort  $HOME/data/dicom/$s/$o
        echo "forwarded object."
        sleep 3
        echo "checking object again."
        isForwarded=`$CheckInsiteScript $s $o | awk '{ print $1 }'`
        if [ "$isForwarded" = "FAILED" ] ; then
           ALLOBJECTSFWD="FALSE"
        fi
        echo -e "isForwarded = $isForwarded for $s $o"
     else
        echo "$o is Ok"
     fi
   done
   if [ "$ALLOBJECTSFWD" = "TRUE" ] ; then
      echo "$s All objects forwarded to Insite"
      echo "$s" >> $SUCCESSFULSENDFILE
   else
      echo "$s SOME OBJECTS still Missing on Insite"
      echo "$s" >> $STILLMISSINGFILE
   fi
done

