#!/bin/bash
# Given two input files create three output files:
# one file which contains lines that are in both
# one file which contains lines only in file 1
# one file which contains lines only in file 2

[ -z "$1" ] && echo -e "\nERROR: Missing parameter file 1.\n" && exit 1
[ -z "$2" ] && echo -e "\nERROR: Missing parameter file 2.\n" && exit 2

if [ ! -e "$1" ] ; then
        echo -e "\nERROR: $1 does not exist\n"
        exit 3
fi

if [ ! -e "$2" ] ; then
        echo -e "\nERROR: $2 does not exist\n"
        exit 4
fi

onlyF1="$1.Only"
onlyF2="$2.Only"
inBoth="$1.$2.inBoth"

echo -e "\nLines only found in $1 will be placed in $onlyF1"
echo -e "Lines only found in $2 will be placed in $onlyF2"
echo -e "Lines found in both $1 and $2 will be placed in $inBoth\n"

while read line
do
    found=`grep -w "$line" $2`
    if [ "$found" = "" ] ; then
        echo "$line" >> $onlyF1
    else
        echo "$line" >> $inBoth.tmp
    fi
done < $1

while read line
do
    found=`grep -w "$line" $1`
    if [ "$found" = "" ] ; then
        echo "$line" >> $onlyF2
    else
        echo "$line" >> $inBoth.tmp
    fi
done < $2

cat $inBoth.tmp | sort -u > $inBoth
rm -f $inBoth.tmp

