#!/bin/bash
################################################################################
#
#  logfix.sh  - fixes logs which are improperly concatenated
#  -  $1   Name of the oper_info logfile which contains multiple days...
#
# +-------------------------------------------------------------------------------+
# | NOTE:  Copy this file to /home/medsrv/var/logs.dated/forever and then run the |
# |        script from there.                                                     |
# | For example:                                                                  |
# |      cp logfix.sh /home/medsrv/var/logs.dated/forever/                        |
# |      cd ~/var/logs.dated/forever/                                             |
# |      ./logfix.sh wcmc.2006-07-16.00:00.oper_info.log.gz                       |
# |                                                                               |
# |      - where wcmc.2006-07-16.00:00.oper_info.log.gz contains the logs for     |
# |        some day in the past up  to but not including 2006-07-16               |
# +-------------------------------------------------------------------------------+
################################################################################

# Static number of days per month lookup table.
nummonthdays[1]=31 #JAN=31
nummonthdays[2]=28 #FEBS=28
nummonthdays[3]=31 #MARS=31
nummonthdays[4]=30 #APRS=30
nummonthdays[5]=31 #MAYS=31
nummonthdays[6]=30 #JUNS=30
nummonthdays[7]=31 #JULS=31
nummonthdays[8]=30 #AUGS=30
nummonthdays[9]=31 #SEPS=31
nummonthdays[10]=31 #OCTS=31
nummonthdays[11]=30 #NOVS=30
nummonthdays[12]=31 #DECS=31

CurrentYear=`date +%Y` # what is the currently the year, i.e. 2007
FirstYear=$CurrentYear # At first we assume the first year in the 
                       # log is the current year
NumYears=1             # We assume there is only log entries from one year

if [ ! $1 ] ; then
 echo "Required Parameter Missing..."
 exit
fi

LHN=`echo $1 | awk -F. '{ print $1 }'` # Log Host name, i.e. wcmc
LDT=`echo $1 | awk -F. '{ print $2 }'` # Log Date, i.e. 2006-07-16
LTM=`echo $1 | awk -F. '{ print $3 }' | tr -d [\\] ` # Log Time, i.e. 00:00

# Build months[] array, i.e. months[2] = "Jun"
echo -e "\n\n\tThe following are the months which are in this concatenated log file...\n"
i=0
for month in $(zcat $1 | awk ' { print $1 } ' | uniq) 
do
  i=$((i+1))
  months[$i]="$month"
  if [ ${months[i]} == "Jan" -a $i -ne 1 ] ; then
    echo -e "\t\tYear Change!"
    NumYears=$((NumYears+1))
  fi
  echo -e "\tMonths[$i] = ${months[i]}"
done

if [ $NumYears -gt 1 ] ; then
   # This log spans more than a single year
   # i.e. has entries from 2006 and 2007
   FirstYear=$((CurrentYear - NumYears + 1))
   echo -e  "\n\t\tThe FirstYear in this log file is: $FirstYear"
fi

BDATE=`date +%F --date="$(zcat $1 | head -n 1 | awk '{ print $1, $2 }') $FirstYear"`
EDATE=`date +%F --date="$(zcat $1 | tail -n 1 | awk '{ print $1, $2 }')"`
BMON=`date +%b --date="$BDATE"`  # Three letter Month name, i.e. Mar
BDAY=`date +%e --date="$BDATE"`  # nonzero padded day, i.e. 3
BBDAY=`date +%d --date="$BDATE"` # zero padded day, i.e. 03
EMON=`date +%b --date="$EDATE"`  # Three letter Month name, i.e. Mar
EDAY=`date +%e --date="$EDATE"`  # nonzero padded day, i.e. 03
BEDAY=`date +%d --date="$EDATE"` # zero padded day, i.e. 03

echo -e "\nLOG SPECIFICATIONS...\n"
echo -e "Log file name Format = $LHN.$LDT.$LTM.<logtype>.gz"
echo -e "Log file contains entries from: $BDATE to $EDATE\n"
##
##  Testing info
##
#echo "LHN = $LHN"
#echo "LDT = $LDT"
#echo "LTM = $LTM"
#echo "Log File BDATE = $BDATE"
#echo "Log File EDATE = $EDATE"
#echo "Log File BBDAY = $BBDAY"
#echo "Log File BDAY = $BDAY"
#echo "Log File BMON = $BMON"
#echo "Log File EDAY = $EDAY"
#echo "Log File EMON = $EMON"
#echo "Log File BEDAY = $BEDAY"


echo -e "\n\n\tDo you really want to Continue? [y/n]"
read ans
if [ "x$ans" == "xn" ] ; then
   exit
fi
##
##  End Testing
##

echo -e "\nBACKING UP ORIGINAL LOGS...\n"
for logtype in "oper_info.log"  "oper_error.log" "access_log" "error_log" "ssl_request_log"
do
   echo "copying $LHN.$LDT.$LTM.$logtype.gz to $LHN.$LDT.$LTM.$logtype.gz.$$.ORIGINAL ... "
   cp $LHN.$LDT.$LTM.$logtype.gz $LHN.$LDT.$LTM.$logtype.gz.$$.ORIGINAL
done

#i=$((i-1))
# echo "i = $i ..."
for logtype in "oper_info.log"  "oper_error.log" "access_log" "error_log" "ssl_request_log"
do
  echo -e "\n\tCurrently Handling $logtype logs..."
  BIGLOG="${LHN}.${LDT}.${LTM}.${logtype}.gz"
  echo -e "\tConcatenated Log is $BIGLOG...\n"
  # Keep up with the current log year
  ThisYear=$FirstYear

  for j in `seq $i` 
  do
     # Keep up with the current log year
     if [ $j -gt 1 -a ${months[j]} == "Jan" ] ; then
        ThisYear=$((ThisYear + 1 ))
     fi

     monthIndex=`date +%m --date="${months[j]} 01"`
     if [ $monthIndex -gt 7  -a $monthIndex -lt 10 ] ; then
        Index=`echo $monthIndex | sed -e 's/0//'`
     else
        Index=$monthIndex
     fi

     if [ $i -gt 1 ] ; then # More than one month
        if [ $j -eq 1 ] ; then # First month
           MBDAY=$BBDAY
           MEDAY=${nummonthdays[Index]}
        elif [  $j -lt $i ] ; then # Niether the first or last month
           MEDAY=${nummonthdays[Index]}
           MBDAY=1
        else  # Last Month
           MEDAY=$BEDAY
           MBDAY=1
        fi
     else # There is only one months worth of data
        MBDAY=$BDAY
        MEDAY=$EDAY
     fi

## More testing
#     echo "j = $j ..."
#     echo "monthIndex = $monthIndex"
#     echo "MBDAY = $MBDAY"
#     echo "MEDAY = $MEDAY"
## End of testing...     

     for day in `seq $MBDAY $MEDAY` 
     do
       CDATE="${months[j]} $((day + 1)) $ThisYear"
       GCDATE=`date +%F --date="$CDATE"`
       OUTFILE="${LHN}.${GCDATE}.00:00.$logtype"

       if [[ -e $OUTFILE.gz && "$OUTFILE.gz" != "$1" ]] ; then
          echo "Moving $OUTFILE.gz to $HOME/work/"
          mv $OUTFILE.gz $HOME/work/
       fi

       echo -e "\t+ Creating $OUTFILE"

       if [[ "x$logtype" == "xaccess_log" || "x$logtype" == "xssl_request_log" ]] ; then
         zcat $BIGLOG | grep "$day/${months[j]}" > $OUTFILE
       else
          if [ $day -lt 10 ] ; then
            zcat $BIGLOG | grep "${months[j]}  $day" > $OUTFILE
          else
            zcat $BIGLOG | grep "${months[j]} $day" > $OUTFILE
          fi
       fi
       echo -e "\t+ \tCompressing $OUTFILE"
       gzip $OUTFILE

     done

  done

done


