#! /bin/bash
# ex: set expandtab shiftwidth=4 softtabstop=4:
#ls -1 /home/medsrv/data/dicom > styiuids.ls
styiuid=$1
unset cleaned
echo "On $(hostname -s)"

function usage
{
  echo -e "\n\nUSAGE:\t$0 STYIUID\n\n"
}

if [ -z "$styiuid" ] ; then
 usage
 exit 1
fi

. ~/etc/setenv.sh
cd $(dirname $0)

MYSQL=$MYSQL_HOME/bin/mysql
DB=imagemedical
STORESTATEVARS="OwnerDevice ProcessMode SourceDevice ArchiveDevice StagedDevices StageState StudyOwner Version ForwardDevices Retrieved"

function isStudyOnWorklist () {
    local styiuid=$1
    unset res
   
    res=$(echo "select styiuid from Dcstudy where styiuid = '$styiuid';" | \
        $MYSQL -N $DB)

    if [ -n "$res" ]; then
        return 0
    else
        return 1
    fi

} # End of isStudyOnWorklist()


function hasObjects() {
    local styiuid=$1
    unset res

    res=$(ls -1 $DICOM_DATA/$styiuid/)

    if [ -n "$res" ]; then
        return 0
    else
        return 1
    fi

} # End of hasObjects()


function isArchived() {
    # returns 0 (success) if archived
    cfStudy=$1
    cfFile=$2
    BDIR=$(pwd)
    [ -z "$cfStudy" -o -z "$cfFile" ] && return 2

    # local checksum
    LMD5=`$DICOM_HOME/bin/objdiff.sh -m "$DICOM_DATA/$cfStudy/$cfFile"`

    # archive checksum
    AMD5=`$BDIR/archivemd5.sh "$cfStudy" "$cfFile"`

    if [ "$AMD5" = "$LMD5" ] ; then
        echo "$cfFile OK"
        return 0
    fi

    echo "$cfFile ERROR [$AMD5]"
    return 1
}
    
function hasTasks() {
    # returns 0 (success) if tasks found (checks parent)
    local styiuid=$1
    unset res

    res=$(./isAnyTaskFor.sh $styiuid)
    if [ "$res" == "NOT FOUND" ]; then
        return 1
    else
        return 0
    fi
} # End of hasTasks()

function getStaged() {
    # Prints list of staged devices (could be an empty list)
    local styiuid=$1

    # unset possible storestate variables here!
#    for tag in $STORESTATEVARS; do
#        unset $tag
#    done
    unset StagedDevices
    . ~/data/dicom/${styiuid}/.info/storestate.rec

    if [ -n "$StagedDevices" ]; then
        echo $StagedDevices
        return 0
    else
        return 1
    fi
#    for tag in $STORESTATEVARS; do
#        [ -n "${!tag}" ] && echo "${indent}${tag} : ${!tag}"
#    done
} # End of getStaged()
    
function getStoreStateTag() {
    # Given a valid storestate tag and styiuid, echo the value of the tag.
    # Return 0 if the tag was set, 1 if the tag was not set.
    local styiuid=$1
    local tagname=$2

    # Check if storestate exists
    [ ! -e ~/data/dicom/"${styiuid}"/.info/storestate.rec ] && return 1
    # Check if tagname is valid and unset all tags we know about
    unset validtag
    for t in $STORESTATEVARS; do
        unset $t
        [ "$tagname" == "$t" ] && validtag=1
    done
    [ -z "$validtag" ] && echo "Critical error: Invalid storestate tag $tagname" && exit 1

    . ~/data/dicom/${styiuid}/.info/storestate.rec
    if [ -n "${!tagname}" ]; then
        echo ${!tagname}
        return 0
    else
        return 1
    fi
} # End of getStoreStateTag()

function getParent() {
    # Prints this server's parent

    res=$(echo "select ID from Target where type = 'P';" | \
        $MYSQL -N $DB)
    echo $res
} # End of getParent()


function getChildren() {
    # Prints a list of this server's children

    res=$(echo "select ID from Target where type = 'C';" | \
        $MYSQL -N $DB)
    echo $res
} # End of getChildren()


function isOnChild() {
    # Returns 0 (success) if the child has a ~/data/dicom dir for this study
    local styiuid=$1
    local child=$2

    childip=$(echo "select HOST from Target where ID = '$child';" | \
        $MYSQL -N $DB)

    res=$(ssh -n $childip ls -d /home/medsrv/data/dicom/$styiuid 2>/dev/null)
    [ -n "$res" ] && return 0 || return 1

} # End of isOnChild()

function hasDcobjectEntry() {
    local styiuid=$1

    res=$(echo "select styiuid from Dcobject where styiuid = '$styiuid';"|~/component/mysql/bin/mysql -N imagemedical)
    [ -n "$res" ] && return 0 || return 1

} # hasDcobjectEntry()


function hasDcreportEntry() {
    local styiuid=$1

    res=$(echo "select styiuid from Dcreport where styiuid = '$styiuid';"|~/component/mysql/bin/mysql -N imagemedical)
    [ -n "$res" ] && return 0 || return 1

} # hasDcreportEntry()


function isOrder() {

    local styiuid=$1
    local numobjs=$(ls -1 /home/medsrv/data/dicom/${styiuid}/ 2> /dev/null | wc -l)
    local ofile
    
    echo "numobjs == $numobjs"
    # More than one object means that this study is not an order
    [ "$numobjs" -ne "1" ] && return 1

    # Check to make sure the file is a PbR
    ofile=$(ls -1 /home/medsrv/data/dicom/${styiuid}/ 2> /dev/null)
    echo "ofile == $ofile"
    [ "${ofile#PbR}" = "$ofile" ] && return 1

    # Check to make sure the PbR is an order
    status=$(${DCMTK_HOME}/bin/dcmdump +P f215,ba02 "${DICOM_DATA}/${styiuid}/${ofile}" | sed 's;^.*\[\(.*\)\].*$;\1;')
    echo "status == $status"

    [ "${status#-}" = "$status" ] && return 1
    return 0
} # isOrder()


function cleanStudy() {
    wd=$(pwd)
    local styiuid=$1
    [ -z "$styiuid" ] && return 1

    echo "Cleaning $styiuid on $(hostname -s)"
    echo "Removing references"
    ./checkrefs.sh -s $styiuid -S -R -c
    
    echo "checking for study directory"
    if [ -d /home/medsrv/data/dicom/"$styiuid" ]; then
        echo "study dir still exists, removing"
        find /home/medsrv/data/dicom/${styiuid}/ -mindepth 1 -follow | xargs rm -rf 
        rm -rf /home/medsrv/data/dicom/${styiuid}
    fi
    echo "Checking for Dcobject entries"
    if hasDcobjectEntry $styiuid; then
        echo "removing Dcobject entries for $styiuid"
        echo "delete from Dcobject where styiuid = '$styiuid';"|~/component/mysql/bin/mysql -N imagemedical
    fi
    echo "Checking for Dcreport entries"
    if hasDcreportEntry $styiuid; then
        echo "removing Dcreport entries for $styiuid"
        echo "delete from Dcreport where styiuid = '$styiuid';"|~/component/mysql/bin/mysql -N imagemedical
    fi
} # cleanStudy()



children=$(getChildren)
parent=$(getParent)    

# Check if study exists
echo -n "exists:"
if ls -d ~/data/dicom/$styiuid 2>/dev/null; then
    echo "yes"
else
    echo "no"
    echo "skipping.."
    exit 2
fi

echo -n "onwl :"
if isStudyOnWorklist $styiuid; then
    echo "yes"
    echo "skipping.."
    exit 2
else
    echo "no"
fi
./checkrefs.sh -s $styiuid -S -R
#echo -n "hastasks :"
#if hasTasks $styiuid; then
#    echo "yes"
#    echo "$styiuid has tasks, skipping.."
#    exit 2
#else
#    echo "no"
#fi

# Check if this server has children
if [ -n "$children" ]; then
    # Check if staged
    stagedlist=$(getStaged $styiuid)
    if [ -n "$stagedlist" -a "$stagedlist" != "$parent" ]; then
        echo "$styiuid marked staged on $stagedlist"
        unset onchild
        for c in $stagedlist; do
            if isOnChild $styiuid $c; then
                [ -n "$onchild" ] && onchild="$onchild $c" || onchild=$c
            fi
        done
        if [ -n "$onchild" ]; then
            echo "$styiuid exists on $onchild"
            exit
        else
            echo "$styiuid does not exist on $children"
        fi 
    else
        echo "$styiuid : Not staged (to a child, anyway)"
    fi
    # Check if I am not owner
    ownerdevice=$(getStoreStateTag $styiuid OwnerDevice)
    if [ -n "$ownerdevice" -a "$ownerdevice" != "$parent" ]; then
        echo "$styiuid has OwnerDevice $ownerdevice"
        if isOnChild $styiuid $ownerdevice; then
            echo "$styiuid: Exists on $ownerdevice"
            exit 2
        else
            echo "$styiuid: Does not exist on $ownerdevice"
        fi
    else
        echo "$styiuid: no OwnerDevice set or ownerdevice == parent"
        echo "ownerdevice == $ownerdevice : parent == $parent"
    fi
fi

echo -n "hasobjects :"
if hasObjects $styiuid; then
    echo "yes"
    # Check if this is an order
    if isOrder $styiuid; then
        echo "$styiuid is an order"
        cleanStudy $styiuid
        cleaned="yes"
    else
        echo "$styiuid is not an order"
        unset archived
        for ofile in $(ls -1 $DICOM_DATA/$styiuid/); do
            if isArchived $styiuid $ofile; then
                archived="yes"
            else
                unset archived
                break
            fi
        done
        echo -n "isarchived :"
        if [ -z "$archived" ]; then
            echo "$styiuid : not all objs archived"
        else
            echo "yes"
            cleanStudy $styiuid
            cleaned="yes"
            ./checkrefs.sh -s $styiuid -S -R
        fi
    fi
else
    echo "no"
    cleanStudy $styiuid
    cleaned="yes"
    ./checkrefs.sh -s $styiuid -R -S
fi

if [ -n "$cleaned" ]; then
    if [ -n "$parent" ]; then
        parentip=$(echo "select HOST from Target where ID = '$parent';" | \
            $MYSQL -N $DB)

        ssh -n $parentip ${PWD}/servercleanup.sh $styiuid
    fi
fi

