#! /bin/bash
# ex: set expandtab shiftwidth=4 softtabstop=4 syntax=sh:

# Set up keys for all dotcom devices attached to me

clean=$1 # Danger, Will Robinson!

sshuser=$(whoami)
thisserver=$(hostname -s)
effuser=$(id -u)
setkeyspath="/home/image/tools/"

if [ -n "$clean" ]; then
    echo "The 'clean' function was sort of intended for testing only ...."
    echo "It will remove ALL keys and authorized_keys from all servers..(including service keys!)"
    echo
    read -p "Are you sure you want to remove ALL keys and authorized_keys from ALL servers? (y/N):" yn
    if [ "$yn" != "y" ]; then
        exit 0
    fi
fi

if [ "$effuser" -eq 0 ]; then
    childdevices=$(su - medsrv -c "echo \"select ID from Target where TYPE='C';\"|/home/medsrv/component/mysql/bin/mysql -N imagemedical")
    parent=$(su - medsrv -c "echo \"select ID from Target where TYPE='P';\"|/home/medsrv/component/mysql/bin/mysql -N imagemedical")
    authorizedkeyfile=/root/.ssh/authorized_keys
    sshdir=/root/.ssh
else
    childdevices=$(echo "select ID from Target where TYPE='C';"|/home/medsrv/component/mysql/bin/mysql -N imagemedical)
    parent=$(echo "select ID from Target where TYPE='P';"|/home/medsrv/component/mysql/bin/mysql -N imagemedical)
    authorizedkeyfile=/home/${sshuser}/.ssh/authorized_keys
    sshdir=/home/medsrv/.ssh
fi

if [ ! -f "$sshdir"/id_rsa.pub -a -z "$clean" ]; then
    echo "This host does not appear to have keys, running ssh-keygen"
    echo "(Hit enter as required for no passphrase)"
    ssh-keygen -t rsa
fi

[ -f "$sshdir"/id_rsa.pub ] && mypubkey=\"$(cat $sshdir/id_rsa.pub)\" 

 
function getIpFromId() {
    local id=$1

    if [ "$effuser" -eq 0 ]; then
        local h=$(su - medsrv -c "echo \"select HOST from Target where ID='$id';\"|/home/medsrv/component/mysql/bin/mysql -N imagemedical")
    else
        local h=$(echo "select HOST from Target where ID='$id';"|/home/medsrv/component/mysql/bin/mysql -N imagemedical)
    fi
    echo $h
} # getIpFromId()


function setUpKeys() {
    local id=$1
    local recurse=$2

    local h=$(getIpFromId $id)
    echo "(Enter root password for $id as required)"

    if [ -z "$clean" ]; then # Global (first arg to script)
        # If .ssh doesn't exist, it's a real bitch to copy stuff to it =)
        echo "Making $sshdir on $id and setting permissions 700, ownership $sshuser.$sshuser"
        ssh root@$h "if [ ! -d \"$sshdir\" ]; then 
                        mkdir -p $sshdir;
                     fi;
                     chown $sshuser.$sshuser $sshdir;
                     chmod 700 $sshdir;"
                
        echo "Copying ${thisserver}'s $sshuser public key to $id ($h) $authorizedkeyfile"
        ssh -t -t root@${h} "echo $mypubkey >> $authorizedkeyfile"

        if [ -n "$recurse" ]; then
            echo "Copying $0 to $id ($h)"
            rsync -a -e ssh $0 root@${h}:${setkeyspath}
            echo "Running $0 on $id ($h) as $sshuser"
            ssh -t -t $h $0
        fi
    else
        # clean was set
        # Recurse first, so that we don't have to enter passwords a ton of times
        # "recurse" is only specified for $thisserver's children.
        # Handle removing my own keys just before exit
        if [ -n "$recurse" ]; then
            echo "Copying $0 to $id ($h)"
            rsync -a -e ssh $0 root@${h}:${setkeyspath}
            echo "Running '$0 clean' on $id ($h) as root"
            ssh -t -t root@$h $0 clean
        fi
        
    fi
} #setUpKeys()

# ----Main----
  
cd

for id in $childdevices; do
    # $childdevices is a list of child hostids (target table)
    echo
    setUpKeys $id recursive
    echo
done

if [ -n "$parent" -a -z "$clean" ]; then 
    # Can only be one parent, $parent contains it's id
    # If "clean" was specified, don't clean the parent's keys (let the parent do that itself)
    # Don't ever recurse on parent, unless you want to send the system into an infinite loop =)
    echo
    setUpKeys $parent 
    echo
elif [ -n "$clean" ]; then
    echo "removing root and medsrv keys on $thisserver"
    rm -rf /root/.ssh /home/medsrv/.ssh
fi
echo "Done on $thisserver"

