#! /usr/bin/python

""" Tool for displaying/manipulating references and storestate.rec """

import os
import dialog

class DotcomStudy:

    "Store study level dotcom metadata"

    gethashdir_bin = "~/component/utils/bin/getHashDirectory"
    ref_bin = "~/component/utils/bin/refcount"
    sstate_bin = "~/component/utils/bin/storestate"

    dicom_dir = "/home/medsrv/data/dicom.hashed"

    def __init__(self, styiuid=None):
        self.styiuid = styiuid
        hashdir = self.findHashdir()
        self.objdir = None
        self.storestate = {}
        if os.path.isdir("%s/%s" % (self.dicom_dir, hashdir)):
            self.objdir = "%s/%s" % (self.dicom_dir, hashdir)
            self.storestate["ssfile"] = "%s%s" % (self.objdir, ".info/storestate.rec")
            self.readStorestate(self.storestate)
        self.STUDYDIR = self.readRefs("STUDYDIR")
        self.STUDYDB = self.readRefs("STUDYDB")
        self.objectlist = self.initObjects()
    # __init__()

    def readStorestate(self, ssdict):
        for line in open(ssdict["ssfile"]):
            kvpair = line.split("=") # .split returns a list. element 0 is tag, element 1 is value
            kvpair[1] = kvpair[1][:-1] # Strip off linefeed
            ssdict[kvpair[0]] = kvpair[1]
    # readStorestate()

    def findHashdir(self):
        args = "-i"
        cmd = "%s %s %s" % (self.gethashdir_bin, args, self.styiuid)
        hashdir = os.popen(cmd).readline()
        hashdir = hashdir[:-1]
        return hashdir
    # findHashdir()

    def readRefs(self, reftype):
        args = "-i %s -p -t %s" % (self.styiuid, reftype)
        cmd = ". ~/etc/setenv.sh; %s %s" % (self.ref_bin, args)
        retval = os.popen(cmd).readlines()
        retval = stripLinefeeds(retval)
        return retval
    # readRefs()

    def getStorestate(self):
        return self.storestate
    # getStorestate()

    def getObjects(self):
        return self.objectlist
    # getObjects()

    def getStudydirRefs(self):
        return self.STUDYDIR
    # getStudydirRefs()

    def getStudydbRefs(self):
        return self.STUDYDB
    # getStudydbRefs()

    def initObjects(self):
        """
        Get the files in data/dicom/<styiuid>
        Create a (dicom) object instance for each file
        which will have the OBJECTFILE refs
        return the list of instances
        """
        if not self.objdir:
            return None 
        objects = []
        dirlisting = os.listdir(self.objdir)
        for f in dirlisting:
            if os.path.isfile(os.path.join(self.objdir, f)):
                obj = DicomObject(self.styiuid, f)
                objects.append(obj)
        return objects
    # initObjects()

    def doCmd(self, cmd, *args):
        method = getattr(self, cmd)
        if callable(method): return method(*args)
    # doCmd()

# Class DotcomStudy                


          
class DicomObject:

    """ Contains dotcom info for dicom objects """

    ref_bin = "~/component/utils/bin/refcount"

    def __init__(self, styiuid, sopiuid):
        self.styiuid = styiuid
        self.name = sopiuid
        self.references = self.readRefs(self.styiuid, self.name)
        self.tasks = []
    # __ init__()

    def readRefs(self, styiuid, sopiuid):
        args = "-i %s/%s -p -t OBJECTFILE" % (styiuid, sopiuid)
        cmd = ". ~/etc/setenv.sh; %s %s" % (self.ref_bin, args)
        retval = os.popen(cmd).readlines()
        retval = stripLinefeeds(retval)
        return retval
    # initRefs()

    def getRefs(self):
        return self.references
    # getRefs()

    def getName(self):
        return self.name
    # getName()

# class DicomObject

    
def stripLinefeeds(lines):
    retval = []
    for line in lines:
        line = line[:-1]
        retval.append(line)
    return retval
# stripLinefeeds()


def populateMenu(values):
    """ values should be a list of tuples containing
        two entries, tag and text"""

def showMenu(menu):
    pass    
        
# Main
def main():

    """
    todo:

    Allow browsing or searching DB for study to operate on

    Set up a real event handler that keeps track of what piece of the program
    the user is in.

    menu= Main (pick a study)
          Study (List Objects, STUDYDB refs, STUDYDIR refs, tasks, storestate, back to Main)
          Object (List OBJECTDIR refs, tasks, dicom attributes, back to Study)
          Reference (Edit, Back  (last menu))
          Storestate (Edit, Back)
          Task (Info, Reschedule, Clean, Reprioritize, Suspend)
          
          

    More stuff edit/change references, storestates, etc
    """



    curmenu = "Main"
    prevmenu = "None"



    a = DotcomStudy("1.2.392.200036.9116.2.2.2.1762553087.996200635.730098")
    cmd = dialog.menu("Study : % s" % a.styiuid, "Use the arrow keys to choose an option", 0, 0, 10, [ 
                ("getObjects", "Display object list"), 
                ("getStudydirRefs", "Display STUDYDIR refs"),
                ("getStudydbRefs", "Display STUDYDB refs")
                 ])
    #print cmd
    if cmd:
	res = a.doCmd(cmd)
	for item in res:
	    print item

    #dialog.textbox("Test", 0, 0, "dialog.py")

main()



"""
    for ref in ("STUDYDB", "STUDYDIR"):
	a.refPrint(ref)
	print

    print "Dicom dir: %s" % a.objdir

    print "Object list:"
    for sobj in a.objectlist:
	n = sobj.getName()
	r = sobj.getRefs()
	print
	print "Object name: %s" % n
	print "References: %s" % r

    for k, v in a.storestate.iteritems():
	print "Tag : %s = %s" % (k, v)

From debugging classes:
    def refPrint(self, reftype):
        print "%s references:" % (reftype)
        refs = getattr(self, reftype)
        for ref in refs:
            print ref
    # refPrint()

    def objsPrint(self):
        print "Object listing"
    # objsPrint()        


Display menu:

List objects : (submenu)
    Show references for an object
    Edit references on an object
    Show tasks for an object

List studies? Filter?
"""
