#! /bin/bash

sourcefile=$1
targetdir=$2
debug=1
MEDUSR="medsrv"
MYSQL_HOME="/home/medsrv/component/mysql"
OTHERSERVERS=""
OTHERSERVERSFILE="otherservers"

[ -f "$OTHERSERVERSFILE" ] && . $OTHERSERVERSFILE

# Query info about parent or child servers
PARENT=`su - $MEDUSR -c "$MYSQL_HOME/bin/mysql -N -B -e 'select HOST, TYPE from Target;' imagemedical|awk -F'\t' '{ if ( \\$2 == \"P\" ) print \\$1; }'"`
CHILDREN=`su - $MEDUSR -c "$MYSQL_HOME/bin/mysql -N -B -e 'select HOST, TYPE from Target;' imagemedical|awk -F'\t' '{ if ( \\$2 == \"C\" ) print \\$1; }'"`

serverlist=$(echo -n $PARENT; echo -n " "; echo -n $CHILDREN; echo -n " "; echo $OTHERSERVERS)

[ -n "$PARENT" ] && echo "Info: Parent server(s) found: $PARENT."

[ -n "$CHILDREN" ] && echo "Info: Child server(s) found: `for e in $CHILDREN; do echo -n [$e] ;done`."
[ -n "$OTHERSERVERS" ] && echo "Info: Other server(s) found: `for e in $OTHERSERVERS; do echo -n [$e] ;done`."

if [ -z "$sourcefile" -o -z "$targetdir" ]; then
    echo "No file specified or no destination directory specified! Nothing to do!"
    exit 1
fi

[ -n "$serverlist" -a -n "$debug" ] && echo "Servers to synchronize: $serverlist"
echo "Press enter to continue synchronizing $sourcefile to all servers in the dotcom."
echo "or Control-C to abort."

read a

for target in $serverlist; do
    echo "Synchronizing $target..."
    rsync -va -e ssh --progress $sourcefile $target:$targetdir
    if [ $? -ne 0 ]; then
        echo "rsync returned with error $?"
        echo "Problem on host $target"
        problemlist=$(echo $problemlist $target)
    else
        echo
        echo "$sourcefile has been synchronized on host $target:$targetdir."
        echo
    fi
done

if [ -n "$problemlist" ]; then
    echo "ALERT!"
    echo "Problems occurred while synchronizing the following servers:"
    echo -e $problemlist|sed 's/ /\n/g'
    echo
    echo "Check that the server's IP is correct in the device table and that"
    echo "the target is pingable"
    echo
    echo "Sychronization completed with errors."
fi

